PlotEquiMap <- function(var, lon, lat, filled.continents = TRUE, 
                        drawleg = TRUE, greenwich = NULL, map_pars = NULL) {
  if (!(is.numeric(var) && (length(dim(var)) == 2))) {
    stop("Error: parameter 'var' should be a numeric matrix.")
  }
  if (!(is.numeric(lon) && (length(dim(lon)) <= 1)) || any(is.na(lon))) {
    stop("Error: parameter 'lon' should be a numeric vector without NA values.")
  }
  if (!(is.numeric(lat) && (length(dim(lat)) <= 1)) || any(is.na(lat))) {
    stop("Error: parameter 'lat' should be a numeric vector without NA values.")
  }
  if (!all(dim(var) == c(length(lat), length(lon)))) {
    stop("Error: parameters 'lon' and/or 'lat' don't match the size of 'var'.")
  }
  if (length(lon) < 2 || length(lat) < 2) {
    stop("Error: at least a 2 by 2 region should be provided.")
  }

  if (!(is.logical(drawleg) || (drawleg %in% c('E', 'S')))) {
    stop("Error: parameter 'drawleg' should be TRUE, FALSE, 'E' or 'S'.")
  }
  if (!is.null(greenwich)) {
    if (!is.logical(greenwich)) {
      stop("Error: 'greenwich' must be NULL, TRUE or FALSE.")
    }
  }
  default_map_pars <- list(interior = FALSE, fill = filled.continents, 
                           wrap = TRUE, resolution = 0)
  if (!is.null(map_pars)) {
    if (!is.list(map_pars)) {
      stop("Error: 'map' pars should be a list of parameters to the function 'maps::map'.")
    }
    default_map_pars[names(map_pars)] <- map_pars
  }
  map_pars <- default_map_pars

  # Keep the original device parameters
  old_par <- par(no.readonly = TRUE)

  # Reorder the longitudes and latitudes
  lon_reord <- sort(lon, index.return = TRUE)
  lon <- lon_reord$x
  lon_reord <- lon_reord$ix
  first_lon <- lon[1]
  last_lon <- tail(lon, 1)
  if (first_lon > 360 || last_lon > 360 || first_lon < -360 || last_lon < -360) {
    stop("Error: all 'lon' and 'lat' values must be in the range [-360, 360].")
  }
  if (last_lon - first_lon >= 360) {
    stop("Error: max(lon) - min(lon) must be smaller than 360 (i.e. there can't be overlapping values).")
  }
  lat_reord <- sort(lat, index.return = TRUE)
  lat <- lat_reord$x
  lat_reord <- lat_reord$ix
  first_lat <- lat[1]
  last_lat <- tail(lat, 1)
  if (first_lat > 90 || last_lat > 90 || first_lat < -90 || last_lat < -90) {
    stop("Error: all 'lon' and 'lat' values must be in the range [-360, 360].")
  }

  # Work out the best arrangement for legend
  requested_width <- last_lon - first_lon
  requested_height <- last_lat - first_lat
  display_width <- par('fin')[1]
  display_height <- par('fin')[2]
  if (drawleg != FALSE) {
    if ((requested_height/requested_width > display_height/requested_width) && 
        (drawleg != 'S')) {
      layout(matrix(1:2, 1, 2), widths = c(5, 1))
    } else {
      layout(matrix(1:2, 2, 1), heights = c(5, 1))
    }
  }

  if (is.null(greenwich)) {
    # Detect automatically whether the data is across greenwich
    lon_gaps <- lon - c(tail(lon, 1) - 360, lon[-length(lon)])
    greenwich_gap <- lon_gaps[1]
    if (greenwich_gap < min(lon_gaps[-1])) {
      print("Warning: irregular gap detected between last and first longitudes in 'lon'. If 'lon' is irregular, automatic detection of greenwich may be wrong. Use parameter 'greenwich' instead.")
    }
    if (((tail(lon, 1) - head(lon, 1)) / min(lon_gaps[-1])) > length(lon)) {
      across_green <- TRUE
    } else {
      across_green <- FALSE
    }
    greenwich <- across_green
  }
  # Arrange data in function of if plot across greenwich is requested or not
  # and select the corresponding map.
  if (greenwich) {
    lon_reord <- c(lon_reord[which(lon >= 180)], 
                   lon_reord[which((lon >= -180) & (lon < 180))],
                   lon_reord[which(lon < -180)])
    lon <- c(lon[which(lon >= 180)], lon[which((lon >= -180) & (lon < 180))], 
             lon[which(lon < -180)])
    lon[which(lon >= 180)] <- lon[which(lon >= 180)] - 360
    lon[which(lon < -180)] <- lon[which(lon < -180)] + 360
    map_to_draw <- 'world'
  } else {
    lon_reord <- c(lon_reord[which(lon >= 0)], lon_reord[which(lon < 0)])
    lon[which(lon < 0)] <- lon[which(lon < 0)] + 360
    map_to_draw <- 'world2'
  }

  # Reorder the input matrix
  var[, 1:dim(var)[2]] <- var[, lon_reord]
  var[1:dim(var)[1], ] <- var[lat_reord, ]
  image(as.vector(lon), as.vector(lat), t(var))
  #data(coastmap, package = 'GEOmap', envir = environment())
  #coastmap$STROKES$col <- rep('black', length(coastmap$STROKES$col))
  #GEOmap::plotGEOmap(coastmap, MAPstyle = 2, add = TRUE)
  # First get the map data
  map_pars[c('database', 'add', 'plot')] <- list(database = map_to_draw, 
                                                 add = FALSE, plot = FALSE)
  world <- do.call(map, map_pars)
  # Then amend it as needed
  # If geenwich, if filled TRUE or FALSE in both cases, 
  # points beyond -180 and 180 have to be moved
  fixLons <- function(world, fun, thr, shift, fixAnt) {
    splitByNA <- function(x) {
      ids <- 1 + cumsum(is.na(x))
      not.na <- !is.na(x)
      split(x[not.na], ids[not.na])
    }
    mergeByNA <- function(x) {
      as.vector(unlist(sapply(x, function(y) c(y, NA))))
    }
    lands_x <- splitByNA(world$x)
    lands_y <- splitByNA(world$y)
    lands <- world$names
    lands <- c(lands, rep("", length(lands_x) - length(lands)))
    lands_to_remove <- c()
    for (land in 1:length(lands)) {
      bad_lons <- which(fun(lands_x[[land]], thr))
      if (length(bad_lons) > 0) {
        new_land_n <- 1
        previous_lon <- NULL
        first_land_lon <- bad_lons[1]
        for (bad_lon in bad_lons) {
          if (!is.null(previous_lon)) {
            if ((bad_lon - previous_lon) > 1) {
              lands_x <- c(lands_x, list(lands_x[[land]][first_land_lon:previous_lon] - shift))
              lands_y <- c(lands_y, list(lands_y[[land]][first_land_lon:previous_lon]))
              lands <- c(lands, paste0(lands[land], '_', new_land_n))
              lands_x[[land]] <- lands_x[[land]][-c(first_land_lon:previous_lon)]
              lands_y[[land]] <- lands_y[[land]][-c(first_land_lon:previous_lon)]
              new_land_n <- new_land_n + 1
              first_land_lon <- bad_lon
            }
          }
          previous_lon <- bad_lon
        }
        lands_x <- c(lands_x, list(lands_x[[land]][first_land_lon:bad_lon] - shift))
        lands_y <- c(lands_y, list(lands_y[[land]][first_land_lon:bad_lon]))
        lands <- c(lands, paste0(lands[land], '_', new_land_n))
        lands_x[[land]] <- lands_x[[land]][-c(first_land_lon:bad_lon)]
        lands_y[[land]] <- lands_y[[land]][-c(first_land_lon:bad_lon)]
        if (length(lands_x[[land]]) == 0) {
          lands_to_remove <- c(lands_to_remove, land)
        }
      }
    }
    if (length(lands_to_remove) > 0) {
      lands_x <- lands_x[-lands_to_remove]
      lands_y <- lands_y[-lands_to_remove]
      lands <- lands[-lands_to_remove]
    }
    world$x <- mergeByNA(lands_x)
    world$y <- mergeByNA(lands_y)
    world$names <- lands
    world$range <- c(min(world$x, na.rm = TRUE), max(world$x, na.rm = TRUE),
                     min(world$y, na.rm = TRUE), max(world$y, na.rm = TRUE))
    world
  }
  if (greenwich) {
    world <- fixLons(world, `>`, 180, -360, filled.continents)
    world <- fixLons(world, `<`, -180, 360, FALSE)
  } else {
    world <- fixLons(world, `>`, 360, -360, filled.continents)
    world <- fixLons(world, `<`, 0, 360, FALSE)
  }
  # Finally plot the map
  map_pars[c('database', 'add', 'plot')] <- list(database = world, 
                                                 add = TRUE, plot = TRUE)
  do.call(map, map_pars)

  # Restore the original device parameters
  par(old_par)
}
